package com.uas.ps.inquiry.dao;

import com.uas.ps.inquiry.model.InquiryRemind;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.math.BigInteger;
import java.util.Date;
import java.util.List;

/**
 * 询价推荐表数据库操作
 *
 * Created by hejq on 2018-01-31.
 */
@Repository
public interface InquiryRemindDao extends JpaRepository<InquiryRemind, Long>, JpaSpecificationExecutor<InquiryRemind> {

    /**
     * 通过vendUU和公共询价明细id更新该单据的状态
     *
     * @param sourceId 公共询价明细id
     * @param value    单据状态
     *                 <note>
     *                 {@code 100 在录入}
     *                 {@code 101 已提交}
     *                 {@code 102 已审核}
     *                 {@code 103 未通过}
     *                 {@code 104 已通过}
     *                 </note>
     * @param vendUU   被推荐的企业
     */
    @Transactional
    @Modifying
    @Query("update InquiryRemind set status = :value where itemId = :sourceId and vendUU = :vendUU")
    void updateStatus(@Param("sourceId") Long sourceId, @Param("value") int value, @Param("vendUU") Long vendUU);

    /**
     * 通过vendUU和公共询价明细id更新该单据的状态
     *
     * @param sourceId 公共询价明细id
     * @param value 单据状态
     *              <note>
     *                  {@code 100 在录入}
     *                  {@code 101 已提交}
     *                  {@code 102 已审核}
     *                  {@code 103 未通过}
     *                  {@code 104 已通过}
     *              </note>
     * @param vendUU 被推荐的企业
     */
    @Transactional
    @Modifying
    @Query("update InquiryRemind set status = :value,newId = :id where itemId = :sourceId and vendUU = :vendUU")
    void updateStatus(@Param("id") Long id, @Param("sourceId") Long sourceId, @Param("value") int value, @Param("vendUU") Long vendUU);

    /**
     * 通过被推荐的企业UU，个人UU，来源id判断单据是否存在
     *
     * @param vendUserUU 推荐的企业UU
     * @param vendUU     个人UU
     * @param itemId     来源id
     */
    List<InquiryRemind> findByVendUserUUAndVendUUAndItemId(Long vendUserUU, Long vendUU, Long itemId);

    /**
     * 通过发布企业UU，询价明细id查询商机list
     *
     * @param enUU     发布询价企业UU
     * @param itemId     来源id
     * return 商机list
     */
    List<InquiryRemind> findByItemIdAndEnUU(Long itemId, Long enUU);

    /**
     * 查出昨晚17点到今早九点的询价信息
     *
     * @param venduseruu
     * @param venduu
     */
    @Query(value = "select * from purc$inquiry$remind where ir_date > date_add(now(),interval -16 HOUR) and ir_date < now() and ir_venduseruu =?1 and ir_venduu = ?2 and ir_enuu != ir_venduu", nativeQuery = true)
    List<InquiryRemind> findInquiryRemind(BigInteger venduseruu, BigInteger venduu);

    /**
     * 查出今早9点到14点的询价信息
     *
     * @param venduseruu
     * @param venduu
     */
    @Query(value = "select * from purc$inquiry$remind where ir_date > date_add(now(),interval -5 hour) and ir_date < now() and ir_venduseruu =?1 and ir_venduu = ?2  and ir_enuu != ir_venduu ", nativeQuery = true)
    List<InquiryRemind> findInquiryRemind1(BigInteger venduseruu, BigInteger venduu);

    /**
     * 查出14点到17点的询价信息
     *
     * @param venduseruu
     * @param venduu
     */
    @Query(value = "select * from purc$inquiry$remind where ir_date > date_add(now(),interval -3 hour) and ir_date < now() and ir_venduseruu =?1 and ir_venduu = ?2 and ir_enuu != ir_venduu", nativeQuery = true)
    List<InquiryRemind> findInquiryRemind2(BigInteger venduseruu, BigInteger venduu);

    /**
     * 找到询价推荐表中所有的用户
     */
    @Query(value = "select ir_venduseruu from purc$inquiry$remind group by ir_venduseruu", nativeQuery = true)
    List<BigInteger> findvenduseruu();

    /**
     * 找到询价推荐表中的所有公司
     */
    @Query(value = "select ir_venduu from purc$inquiry$remind group by ir_venduu", nativeQuery = true)
    List<BigInteger> findvenduu();

    /**
     * 可配置测试接口
     * @param startTime
     * @param endTime
     * @param useruu
     * @param enuu
     * @return
     */
    @Query(value = "select * from purc$inquiry$remind where ir_date between ?1 and ?2 and ir_venduseruu =?3 and ir_venduu = ?4",nativeQuery = true)
    List<InquiryRemind> findTestInfo(Date startTime, Date endTime, Long useruu, Long enuu);

    /**
     * 根据itemId 查询企业商机
     * @param id  询价明细id
     * @return
     */
    List<InquiryRemind> findByItemId(Long id);
}
