package com.uas.ps.inquiry.dao;

import com.uas.ps.inquiry.model.PublicInquiryItem;
import com.uas.ps.inquiry.model.PurcInquiryItem;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.util.Date;
import java.util.List;

/**
 * 公共询价明细的操作
 *
 * Created by hejq on 2018-01-14.
 */
@Repository
public interface PurcInquiryItemDao extends JpaSpecificationExecutor<PurcInquiryItem>, JpaRepository<PurcInquiryItem, Long> {

    /**
     * 通过关联主表id更新询价明细状态
     *
     * @param sourceId 主表id
     */
    @Transactional
    @Modifying(clearAutomatically = true)
    @Query("update PurcInquiryItem set overdue = 1 where inquiry.id = :sourceId")
    void updateStatus(@Param("sourceId") Long sourceId);

    /**
     * 通过id维护该询价明细客户报价数据
     *
     * @param sourceId
     * @param amount
     */
    @Transactional
    @Modifying
    @Query("update PurcInquiryItem set offerAmount = coalesce(offerAmount, 0) + :amount where id = :sourceId")
    void updateAmount(@Param("sourceId") Long sourceId, @Param("amount") Integer amount);

    /**
     * 通过询价主表id查询包含的明细
     *
     * @param inid 询价主表id
     * @return
     */
    @Query("from PurcInquiryItem i where i.inquiry.id = :inid")
    List<PurcInquiryItem> findByInid(@Param("inid") Long inid);

    /**
     * 查询截止日期
     *
     * @param
     * @return
     */
    @Query("FROM PublicInquiryItem where DATE_FORMAT(endDate, '%y-%m-%d') = DATE_FORMAT(now(), '%y-%m-%d') And agreed is null")
    List<PublicInquiryItem> findendDate();

    /**
     * 查询某个时间段的询价数量
     * @param startDate
     * @param endDate
     * @return
     */
    @Query(value = "select count(*) from purc$puinquiryitems where id_fromdate <= :endDate && id_fromdate >= :startDate", nativeQuery = true)
    Integer countInquiryAmount(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 查询某个时间段的用户数量
     * @param startDate
     * @param endDate
     * @return
     */
    @Query(value = "select COUNT(DISTINCT(id_usertel)) from purc$puinquiryitems where id_fromdate <= :endDate && id_fromdate >= :startDate", nativeQuery = true)
    Integer countInquiryUserAmount(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 查询个人不同型号发布询价数量统计
     *
     * @param enuu      企业uu
     * @param useruu    用户uu
     * @param starttime 开始时间
     * @param endtime   结束时间
     * @return 查询结果
     */
    @Query(value = "select count(1) from (select 1 from purc$puinquiryitems where id_enuu = :enuu and id_useruu = :useruu and id_date between :starttime and :endtime group by id_inbrand, id_cmpcode) a", nativeQuery = true)
    Integer sumPersonalPublish(@Param("enuu") Long enuu, @Param("useruu") Long useruu, @Param("starttime") Date starttime, @Param("endtime") Date endtime);

    /**
     * 查询个人账号 不同型号发布询价数量统计
     *
     * @param useruu    用户uu
     * @param starttime 开始时间
     * @param endtime   结束时间
     * @return 查询结果
     */
    @Query(value = "select count(1) from (select 1 from purc$puinquiryitems where id_enuu is null and id_useruu = :useruu and id_date between :starttime and :endtime group by id_inbrand, id_cmpcode) a", nativeQuery = true)
    Integer sumPersonalPublishWithoutEnuu(@Param("useruu") Long useruu, @Param("starttime") Date starttime, @Param("endtime") Date endtime);


    /**
     * 查询企业不同型号发布询价数量统计
     *
     * @param enuu      企业uu
     * @param starttime 开始时间
     * @param endtime   结束时间
     * @return 查询结果
     */
    @Query(value = "select count(1) from (select 1 from purc$puinquiryitems where id_enuu = :enuu and id_date between :starttime and :endtime group by id_inbrand, id_cmpcode) a", nativeQuery = true)
    Integer sumEnterprisePublish(@Param("enuu") Long enuu, @Param("starttime") Date starttime, @Param("endtime")Date endtime);

    /**
     * 查询个人报价询价数量统计
     *
     * @param enuu      企业uu
     * @param useruu    用户uu
     * @param starttime 开始时间
     * @param endtime   结束时间
     * @return 查询结果
     */
    @Query(value = "select count(1) from public$inquiryitems where id_venduu = :enuu and id_venduseruu = :useruu and id_offertime between :starttime and :endtime", nativeQuery = true)
    Integer sumPersonalReply(@Param("enuu") Long enuu, @Param("useruu") Long useruu, @Param("starttime") Date starttime, @Param("endtime") Date endtime);

    /**
     * 查询企业回复询价数量统计
     *
     * @param enuu      企业uu
     * @param starttime 开始时间
     * @param endtime   结束时间
     * @return 查询结果
     */
    @Query(value = "select count(1) from public$inquiryitems where id_venduu = :enuu and id_offertime between :starttime and :endtime", nativeQuery = true)
    Integer sumEnterpriseReply(@Param("enuu") Long enuu, @Param("starttime") Date starttime, @Param("endtime")Date endtime);

    /**
     * 根据上周最后注册用户UU获取老用户询价数量
     * @param userUU  上周最后注册用户uu
     * @param dateParams 日期
     * @return 查询结果
     */
    @Query(value = "select count(1) from (select 1 from purc$puinquiryitems where date(id_date) = :dateParams and id_useruu <= :userUU group by id_inbrand, id_cmpcode) a", nativeQuery = true)
    Integer sumOldInquiryCountByUserUUAndDate(@Param("userUU") Integer userUU, @Param("dateParams") String dateParams);

    /**
     * 根据日期内获取所有询价数量
     * @param dateParams 日期
     * @return 查询结果
     */
    @Query(value = "select count(1) from (select 1 from purc$puinquiryitems where date(id_date) = :dateParams group by id_inbrand, id_cmpcode) a", nativeQuery = true)
    Integer sumInquiryCountByDate(@Param("dateParams") String dateParams);

    /**
     * 日期区间内，老客户的询价数量
     * @param startDate 开始时间 yyyy-MM-dd
     * @param endDate 结束时间 yyyy-MM-dd
     * @param userUU  上周注册用户最大UU
     * @return 查询结果
     */
    @Query(value = "select count(1) from (select 1 from purc$puinquiryitems where date(id_date) between :startDate and :endDate and id_useruu <= :userUU group by id_inbrand, id_cmpcode) a", nativeQuery = true)
    Integer sumOldInquiryCountWithDateScope(@Param("startDate") String startDate, @Param("endDate") String endDate, @Param("userUU") Integer userUU);

    /**
     * 日期区间内，所有询价数量
     * @param startDate 开始时间 yyyy-MM-dd
     * @param endDate 结束时间 yyyy-MM-dd
     * @return 查询结果
     */
    @Query(value = "select count(1) from (select 1 from purc$puinquiryitems where date(id_date) between :startDate and :endDate group by id_inbrand, id_cmpcode) a", nativeQuery = true)
    Integer sumInquiryCountWithTimeScope(@Param("startDate") String startDate, @Param("endDate") String endDate);


    /**
     * 根据企业UU查询询价条数
     * @param enuu 企业UU
     * @return
     */
    @Query(value = "select count(1) from purc$puinquiryitems where id_enuu = :enuu", nativeQuery = true)
    Integer countByEnuu(@Param("enuu") Long enuu);
}
